-- 
-- Script to create H2 tables for persistent usage
--
-- @author Bernard Chambon bernard.chambon@cc.in2p3.fr

-- Memo about type 
--   bigint Mapped to java.lang.Long
--   int Mapped to java.lang.Integer   (-2147483648 to 2147483647)
--   tinyint Mapped to java.lang.Byte  (-128 to 127)
--  
--  About index :
--   It seems not possible to create index inside create table structure
--   otherwise :  syntax error while creating index with table creation
--   solution :   create INDEX after create table     
--
-- memo about convention we use
--   name of table in CamelCase name without ending 's', I mean TreqsTape
--   field in snack_case, I mean tape_name
--   index with explicit name and with  prefix   idx_tablename_... , fk_tablename_...  
--  
--   suffixe _id for field reference other table, I mean 
--     tape_model_id int  then  foreign key (tape_model_id) references TreqsTapeModel(id)
--   name of date with _date as suffixe,  timestamp as time (because timezone support)
--   (timestamp is also the type used for automatic insert | update date)
-- 
--   Not using enum for status since already done at java app level
--   most of string as varchar(32) except for
--   id char(36); filename varchar(256), username varchar(64), password varchar(128) 


-- clear database !
DROP TABLE IF EXISTS TreqsRequest;
DROP TABLE IF EXISTS TreqsUser;
DROP TABLE IF EXISTS TreqsFile;
DROP TABLE IF EXISTS TreqsTape;
DROP TABLE IF EXISTS TreqsTapeModel;

------  About TapeModel ------
--
DROP TABLE IF EXISTS TreqsTapeModel;
CREATE TABLE TreqsTapeModel (
  -- the (automatic) date of insert and update, cool !
 -- created_date TIMESTAMP default CURRENT_TIMESTAMP(),
 -- updated_date TIMESTAMP AS CURRENT_TIMESTAMP() ,

  tape_model_name varchar(32) NOT NULL unique,  

  tape_model_status  varchar(32) not null, 

  -- to control simultaneous staging per tape model
  max_parallel_staging int not null,

  -- to express reading rate, used to compute timeout (in  MB per/s e.g. 200 MB/s for T10K-D)
  reading_rate int not null,

  PRIMARY KEY (tape_model_name)
) DEFAULT CHARSET=utf8;


-- for test, = ok
--delete from TreqsTapeModel ;
--insert into TreqsTapeModel (tape_model_name, tape_model_status,max_parallel_staging, reading_rate ) 
-- values ('T10K-A', 'ENABLED', '10', '123'), ('T10K-B', 'ENABLED', '10', '123') , ('T10K-C', 'ENABLED', '10', '123') , ('T10K-D', 'ENABLED', '10', '123'); 
--insert into TreqsTapeModel (tape_model_name, tape_model_status,max_parallel_staging, reading_rate) values ('T10K-E', 'ENABLED',  '10', '123'); 
--
--update TreqsTapeModel set tape_model_status='DISABLED' where tape_model_name='T10K-E' ;

------  About Tape  ------
DROP TABLE IF EXISTS TreqsTape;
CREATE TABLE TreqsTape (
  -- the (automatic) date of insert and update, cool !
 -- created_date TIMESTAMP default CURRENT_TIMESTAMP(),
 -- updated_date TIMESTAMP AS CURRENT_TIMESTAMP() ,

  tape_name varchar(32) NOT NULL unique ,
  tape_model  varchar(32) ,

  tape_status varchar(32),

  PRIMARY KEY (tape_name),
  CONSTRAINT fk_treqstape_tape_model  FOREIGN KEY (tape_model) 
                                      REFERENCES TreqsTapeModel(tape_model_name)

) DEFAULT CHARSET=utf8;

create INDEX idx_treqstape_name on TreqsTape(tape_name);
create INDEX idx_treqstape_status on TreqsTape(tape_status);

-- for test, = ok
--delete from TreqsTape ;
--insert into TreqsTape (tape_name, tape_model, tape_status) 
-- values ('TapeNr0' , select tape_model_name from TreqsTapeModel where (tape_model_name = 'T10K-D'), 'ENABLED' ) ; 
--insert into TreqsTape (tape_name, tape_model, tape_status) 
--values ('TapeNr1', 
--        select tape_model_name from TreqsTapeModel where (tape_model_name = 'T10K-D'),
--        'DISABLED' ); 


------  About File  ------
DROP TABLE IF EXISTS TreqsFile;
CREATE TABLE TreqsFile (
  -- the (automatic) date of insert and update, cool !
 -- created_date TIMESTAMP default CURRENT_TIMESTAMP(),
 -- updated_date TIMESTAMP AS CURRENT_TIMESTAMP() ,

  filename varchar(256) NOT NULL unique ,
  filesize bigint ,
  position_on_tape int ,
  offset_position_on_tape bigint,

  -- we do not qualified as 'not null' to allow TreqsFile inserted without tapename
  -- e.g. case a request on a non existing file  
  -- more, this is mandatory for treqs logic, request will be inserted, but HPSS 
  -- metadata will be quireied later
  tape  varchar(32),

  file_status varchar(32),
  file_sub_status varchar(32) default null,

  dispatching_date TIMESTAMP default NULL,
  start_staging_date TIMESTAMP default NULL,
  end_staging_date   TIMESTAMP default NULL,
  ended_date TIMESTAMP default NULL,
  error_message varchar(256) default NULL,

  -- will be generated by app (uuid)
  staging_run_id char(36) default NULL,   -- 36 as fixed lenth for uuid

  PRIMARY KEY (filename),
  CONSTRAINT fk_treqsfile_tape FOREIGN KEY (tape) REFERENCES TreqsTape(tape_name)
) DEFAULT CHARSET=utf8;

create INDEX idx_treqsfile_filename on TreqsFile(filename);
create INDEX idx_treqsfile_status on TreqsFile(file_status, file_sub_status);

-- for test, = ok
--delete from TreqsFile ;
--insert into TreqsFile (filename, filesize, position_on_tape, offset_position_on_tape, 
--                        tape, 
--                        file_status, file_sub_status) 
--       values ('FileNr0' , 123456, 10, 0 ,  
--		select tape_name from TreqsTape where (tape_name = 'TapeNr0'),
--               'CREATED',  'ALREADYONDISK',
--              ) ; 


------  About User  ------
DROP TABLE IF EXISTS TreqsUser;
CREATE TABLE TreqsUser (
  -- the (automatic) date of insert and update, cool !
 -- created_date TIMESTAMP default CURRENT_TIMESTAMP(),
 --  updated_date TIMESTAMP AS CURRENT_TIMESTAMP() ,

  username  varchar(64) NOT NULL,
  password  varchar(128) not null, 

  user_status varchar(32),

  primary key (username)
) DEFAULT CHARSET=utf8;

--delete from TreqsUser ;
--insert into TreqsUser (username,password ) 
--       values ('UserNr0', HASH('SHA256', STRINGTOUTF8('PasswordNr0'), 1000)) ; 


------  About Request  ------
DROP TABLE IF EXISTS TreqsRequest;
CREATE TABLE TreqsRequest (
  -- will be generated by app (uuid)
  id char(36) not NULL ,   -- 36 as fixed lenth for uuid
 
  owner  varchar(64) NOT NULL,
   
  -- qualified as not null because obviously, there is no requets without file
  file  varchar(256) not null ,

  -- the user for who the request is done : not used for internal usage
  account varchar(32) default null ,
  
  request_status varchar(32) ,
  request_sub_status varchar(32) default null,

  submitted_date TIMESTAMP default NULL, -- will be the same as created_date
  ended_date   TIMESTAMP default NULL,  

  PRIMARY KEY (id),
  CONSTRAINT fk_treqsrequest_file FOREIGN KEY (file) REFERENCES TreqsFile(filename),
  CONSTRAINT fk_treqsrequest_owner FOREIGN KEY (owner) REFERENCES TreqsUser(username)
 -- 
) DEFAULT CHARSET=utf8;

create INDEX idx_treqsrequest_owner on TreqsRequest(owner);
create INDEX idx_treqsrequest_status on TreqsRequest(request_status, request_sub_status);


-- for test, = ok
--delete from TreqsRequest ;
--insert into TreqsRequest(id, 
--                         owner, 
--                         file,
--                         request_status, request_sub_status,
--                          submitted_date
--                         ) 
--       values (RANDOM_UUID(), 
--               select username from TreqsUser where (username = 'UserNr0'),
--               select filename from TreqsFile where (filename = 'FileNr0'),
--               'SUBMITTED',  NULL,
--               now()
--              ) ; 


